package cn.com.qmth.examcloud.app.core.utils;

import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import java.security.SecureRandom;
import java.util.concurrent.TimeUnit;

/**
 * HttpClient
 */
public class HttpClientBuilder {

    private static final Logger LOG = LoggerFactory.getLogger(HttpClientBuilder.class);

    private static OkHttpClient client;

    static {
        client = Client.INSTANCE.getInstance();
    }

    public static OkHttpClient getClient() {
        return client;
    }

    private enum Client {

        INSTANCE;

        Client() {
            instance = new OkHttpClient
                    .Builder()
                    .connectTimeout(15, TimeUnit.SECONDS)
                    .readTimeout(30, TimeUnit.SECONDS)
                    .writeTimeout(30, TimeUnit.SECONDS)
                    .sslSocketFactory(sslSocketFactory(), new TrustAllCert())
                    .hostnameVerifier(new TrustAllHost())
                    .build();
        }

        private OkHttpClient instance;

        public OkHttpClient getInstance() {
            return instance;
        }

        private SSLSocketFactory sslSocketFactory() {
            try {
                final String type = "TLS";
                SSLContext sslContext = SSLContext.getInstance(type);
                sslContext.init(null, new TrustManager[]{new TrustAllCert()}, new SecureRandom());
                return sslContext.getSocketFactory();
            } catch (Exception e) {
                LOG.error(e.getMessage());
                throw new RuntimeException(e);
            }
        }
    }

    public static byte[] download(String url) throws RuntimeException {
        Request request = new Request.Builder().url(url).get().build();

        try (Response response = getClient().newCall(request).execute();) {
            if (response.isSuccessful()) {
                return response.body().bytes();
            }

            throw new RuntimeException("download fail, url is " + url);
        } catch (Exception e) {
            LOG.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }

}